/*
 *  Copyright AIST-CNRS
 *
 *  Authors: Fumio Kanehiro, Florent Lamiraux, Eiichi Yoshida
 */

#ifndef parserOpenHRPKineoDevice_H
#define parserOpenHRPKineoDevice_H

#include "hppOpenHRP/parserOpenHRPKineo.h"
#include "hpp/core/collision-pair.hh"
#include "hpp/model/humanoid-robot.hh"

typedef std::map< unsigned int, unsigned int, std::less<unsigned int> > intMap; 

/************************

CLASS

************************/

/**
  \brief Creates hppDevice from OpenHRP ModelInfo
 */
class CparserOpenHRPKineoDevice : public CparserOpenHRPKineo{
public:
  /**
     Constructor
  */
  CparserOpenHRPKineoDevice(ModelInfo_ptr OpenHRPModelInfo);
  /**
     Destructor
  */
  virtual ~CparserOpenHRPKineoDevice();
  /**
     \brief Creates hppDevice from OpenHRP ModelInfo which is given as a constructor argument
     \param io_device shared pointer to an instance of hpp::model::HumanoidRobot where a joint tree is stored. If io_device is a shared pointer to NULL, an instance of hpp::model::HumanoidRobot is assigned. If a user want to use a specific class which derives hpp::model::HumanoidRobot, an empty(its joint tree is not built) instance of the class can be passed. 
  */
  virtual void parser(hpp::model::HumanoidRobotShPtr& io_device) ;

protected : 

  ktStatus buildHppDevice(hpp::model::HumanoidRobotShPtr& hppDevice) ;

  hpp::model::JointShPtr createHppJoint(const TopenHrpJoint &openHrpJoint, long jointId,  hpp::model::DeviceShPtr hppDevice, const std::string jName);

   /** 
      \brief Create the body stored in info and attach it to KineoWorks joint.
  */
  ktStatus createHppBodyAndAttachToJoint(const TopenHrpJoint& openHrpJoint, long jointId, hpp::model::JointShPtr kppJoint, const std::string jName) ;

  /**
     \brief Convert a vector of dimension 9 into a 3x3 matrix.

     The vector is the concatenation of the matrix rows.
  */
  matrix3d valueMatrixToMatrix3d(ValueMatrix* mat);

  /**
     \brief Extract the rotation matrix from an homogeneous matrix.
  */
  void CkitMat4ToMatrix3d(const CkitMat4& inMat4, matrix3d& outMatrix3d);

  /**
     \brief Computes the coordinates of the Center of Mass in kwsJoint reference frame.

     \param openHRPRelCom position of the center of mass in OpenHRP local frame of the joint.
     \param kwsJoint KineoWorks joint the body is attached to.

     In OpenHRP, the local reference frame of each joint is centered on the origin of the joint and oriented as the global reference frame.
  */
  CkitVect3 kwsTransRelJoint(CkitVect3 &openHRPRelCom, CkwsJointShPtr kwsJoint);

  /**
     \brief Compute the inertia matrix in the local reference frame of the joint.

     \param inOpenHrpInertiaMatrix Inertia matrix expressed in OpenHrp local reference frame
     \param inHppJoint Joint the body is attached to.

     In OpenHRP, the local reference frame of each joint is centered on the origin of the joint and oriented as the global reference frame.
  */
  matrix3d inertiaMatrixInHppJointFrame(const matrix3d& inOpenHrpInertiaMatrix, 
					hpp::model::JointShPtr inHppJoint);
  
 /** 
      \brief Associative array mapping OpenHrp joints to hpp::model::HumanoidRobot joints.

      OpenHrp Joints (class ParserModelObject) are numbered by integer
      ids.  For each HRP2 joint, we build a hpp::model::Joint. In order to
      keep track of the joint association during construction of the
      humanoid kinematic chain, we use an associative array.
  */
  std::map<unsigned int, hpp::model::JointShPtr> jointMap;

  /** 
      \brief Associative array mapping OpenHrp joints to absolute position of corresponding local reference frame.

      Absolute position of OpenHrp joints is not returned in
      ParserModelObject. We need to keep track of the successive frame
      positions of OpenHrp joints in order to correctly build
      hpp::model::Joint.
  */
  std::map<unsigned int, CkitMat4*> jointAbsPosMap;

  /** 
      \brief Associate joint number with link name in linkinfo_hrp2jrl.h
  */
  hpp::core::CollisionPair hrpCP;

  /**
     \brief Get OpenHRPModelInfo
  */
  inline ModelInfo_ptr& openHRPModelInfo() {
    return attOpenHRPModelInfo;
  }

private : 

  /**
     \brief Compute a position matrix moving origin and X axis to given position and orientation
  */
  CkitMat4 positionFromAxisAndCenter(const CkitVect3& inAxis, const CkitPoint3& inCenter);

  ModelInfo_ptr attOpenHRPModelInfo ;

  ParserModelObjectSeq* attOpenHRPModelSeq ;

};

#endif
